using System;
using UnityEngine;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using System.Linq;
using SimpleJSON;
using System.Collections;

namespace ky1001
{
    public class AppearanceLoader : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "ky1001";
        public string pluginName = "AppearanceLoader";
        public string pluginVersion = "1.0";
        public string pluginDate = "2022-05-15";
        public string pluginDescription = "";
        #endregion

        #region Vars
        private const string _vapLookPath = "Custom\\Atom\\Person\\Appearance";
        private const string _jsonLookPath = "Saves\\Person\\appearance";
        private const string _jsonScenePath = "Saves\\scene";
        private string LoadData;
        private string ext;
        private string reload;
        private string loadDir;
        private string vapPreviousFolder;
        private string jsonPreviousFolder;
        private string scenePreviousFolder;
        private string SELF;

        protected JSONStorableStringChooser selectPerson;
        private JSONStorableBool shortcuts_Bool;
        private UIDynamicToggle shortcuts_toggle;

        private JSONStorableBool morphs_Bool;
        private UIDynamicToggle morphs_toggle;
        private JSONStorableBool textures_Bool;
        private UIDynamicToggle textures_toggle;
        private JSONStorableBool eye_Bool;
        private UIDynamicToggle eye_toggle;
        private JSONStorableBool mouth_Bool;
        private UIDynamicToggle mouth_toggle;
        private JSONStorableBool hair_Bool;
        private UIDynamicToggle hair_toggle;
        private JSONStorableBool clothing_Bool;
        private UIDynamicToggle clothing_toggle;
        private JSONStorableBool keepFolder_Bool;
        private UIDynamicToggle keepFolder_toggle;
        private JSONStorableString filename;
        private UIDynamicTextField messageBox;
        private UIDynamicTextField messageBox2;
        private UIDynamicTextField messageBox3;
        private UIDynamicTextField messageBox4;
        private JSONStorableString message;
        private JSONStorableString missingItem;
        private JSONStorableString persons;
        private JSONStorableString hairAndClothing;

        private List<string> personList = new List<string>();
        private JSONNode jSONNode;
        private JSONArray atomArray;
        private JSONArray asArray;
        private JSONArray morphsAsArray;
        private JSONArray hairAsArray;
        private JSONArray clothingAsArray;

        private JSONClass asAtomObject;
        private JSONClass asObject;
        private JSONClass hairAsObject;
        private JSONClass clothingAsObject;
        private JSONClass morphs;

        protected Atom TargetAtom;
        private string id;
        private string characterName;
        private float scale;
        private bool sceneBool;
        private bool reloadBool;
        private string tempHair;
        private string tempClothing;

        private Dictionary<string, List<string>> idDic = new Dictionary<string, List<string>>();
        private Dictionary<string, bool> simClothingDic = new Dictionary<string, bool>();
        private List<string> simClothingList = new List<string>();
        private List<string> idListVap = new List<string>();
        private List<string> hairList = new List<string>();
        private List<string> clothingList = new List<string>();

        JSONStorable geometry;
        DAZCharacterSelector character;
        GenerateDAZMorphsControlUI morphControl;
        Dictionary<string, JSONStorable> paramControls = new Dictionary<string, JSONStorable>();

        private readonly string[] idList = new string[]
        {
            "BreastControl",
            "GluteControl",
            "LowerPhysicsMesh",
            "BreastPhysicsMesh",
            "JawControl",
            "FemaleEyelashes",
            "MaleEyelashes",
            "irises",
            "sclera",
            "lacrimals",
            "skin",
            "textures",
            "teeth",
            "tongue",
            "mouth"
        };

        private readonly string[] idList2 = new string[]
        {
            "rescaleObject",
            "BendFix",
            "FemaleAnatomy",
            "BreastInOut",
            "EyelidControl",
            "AutoJawMouthMorph",
            "AutoExpressions",
            "SoftBodyPhysicsEnabler"
        };

        private readonly string[] idList3 = new string[]
        {
            "AllJointsControl",
            "AnimatedFingerDrive",
            "AppearancePresets",
            "AtomControl",
            "CharacterPoseSnapRestore",
            "DebugJointsControl",
            "Eyes",
            "FemaleAnatomyAlt",
            "FemaleBreastPhysicsPresets",
            "FemaleGlutePhysicsPresets",
            "Genesis2Female",
            "geometry",
            "HairPresets",
            "HeadAudioSource",
            "LeftHandControl",
            "LeftHandFingerControl",
            "lEye",
            "MaleAnatomy",
            "MorphPresets",
            "PhysicsModel",
            "PluginManager",
            "PluginPresets",
            "PosePresets",
            "Preset",
            "rEye",
            "RightHandControl",
            "RightHandFingerControl",
            "SkinPresets",
            "SpeechBubble",
            "ThoughtBubble",
            "TongueControl",
            "TriggerGroup",
            "upperJaw"
        };

        private readonly string[] actionList = new string[]
        {
            "SaveToStore1",
            "SaveToStore2",
            "SaveToStore3",
            "RestoreAllFromStore1",
            "RestorePhysicsFromStore1",
            "RestoreAppearanceFromStore1",
            "RestoreAllFromStore2",
            "RestorePhysicsFromStore2",
            "RestoreAppearanceFromStore2",
            "RestoreAllFromStore3",
            "RestorePhysicsFromStore3",
            "RestoreAppearanceFromStore3",
            "RestoreAllFromDefaults",
            "RestorePhysicalFromDefaults",
            "RestoreAppearanceFromDefaults",
            "CreateUVTemplateTexture",
            "CreateSimTemplateTexture",
            "OpenTextureFolderInExplorer"
        };
        #endregion

        // Use this for initialization
        public override void Init()
        {
            try
            {
                TargetAtom = containingAtom;

                #region Buttons
                filename = new JSONStorableString("Set Appearance Filename", "Custom/Atom/Person/Appearance/");
                vapPreviousFolder = _vapLookPath;
                jsonPreviousFolder = _jsonLookPath;
                scenePreviousFolder = _jsonScenePath;
                SELF = "";

                keepFolder_Bool = new JSONStorableBool("Keep the previous folder", true);
                keepFolder_toggle = CreateToggle(keepFolder_Bool, false);

                var loadVapLook = CreateButton("Load Vap Appearance Preset", false);
                loadVapLook.button.onClick.AddListener(() =>
                {
                    if (!keepFolder_Bool.val)
                    {
                        vapPreviousFolder = _vapLookPath;
                    }
                    filename.val = "";
                    selectPerson.val = "None";
                    sceneBool = false;
                    var vapLookShortcuts = FileManagerSecure.GetShortCutsForDirectory(_vapLookPath, false, false, true, true);
                    SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, "vap", vapPreviousFolder, true, true, false, null, false, vapLookShortcuts);
                });
                JSONStorableAction LoadVapDatas = new JSONStorableAction("Load Appearance from VapPreset", () =>
                {
                    selectPerson.val = "None";
                    sceneBool = false;
                    HandleLoadPreset("");
                });

                var loadJsonLook = CreateButton("Load Json Appearance Preset", false);
                loadJsonLook.button.onClick.AddListener(() =>
                {
                    if (!keepFolder_Bool.val)
                    {
                        jsonPreviousFolder = _jsonLookPath;
                    }
                    filename.val = "";
                    selectPerson.val = "None";
                    sceneBool = false;
                    var jsonLookShortcuts = FileManagerSecure.GetShortCutsForDirectory(_jsonLookPath, false, false, true, true);
                    SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, "json", jsonPreviousFolder, true, true, false, null, false, jsonLookShortcuts);
                });
                JSONStorableAction LoadJsonDatas = new JSONStorableAction("Load Appearance from JsonPreset", () =>
                {
                    selectPerson.val = "None";
                    sceneBool = false;
                    HandleLoadPreset("");
                });

                //var spacer = CreateSpacer(false);
                //spacer.height = 40f;
                //spacer = CreateSpacer(true);
                //spacer.height = 40f;

                var reloadLook = CreateButton("Reload Appearance Preset", true);
                reloadLook.button.onClick.AddListener(() =>
                {
                    reloadBool = true;
                    HandleLoadPreset(reload);
                });
                reloadLook.buttonColor = Color.yellow;
                JSONStorableAction reloadAct = new JSONStorableAction("Reload Appearance Preset", () =>
                {
                    reloadBool = true;
                    HandleLoadPreset(reload);
                });

                shortcuts_Bool = new JSONStorableBool("Use Shortcuts in Scene Load Dialog", false);
                shortcuts_toggle = CreateToggle(shortcuts_Bool, true);

                var selectScene = CreateButton("Select Scene", true);
                selectScene.button.onClick.AddListener(() =>
                {
                    if (!keepFolder_Bool.val)
                    {
                        scenePreviousFolder = _jsonScenePath;
                    }
                    filename.val = "";
                    selectPerson.val = "None";
                    sceneBool = true;
                    var jsonSceneShortcuts = FileManagerSecure.GetShortCutsForDirectory(_jsonScenePath, false, false, true, true);
                    if (!shortcuts_Bool.val)
                    {
                        jsonSceneShortcuts = null;
                    }
                    SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, "json", scenePreviousFolder, true, true, false, null, false, jsonSceneShortcuts);
                });
                JSONStorableAction SelectSceneDatas = new JSONStorableAction("Select Scene", () =>
                {
                    selectPerson.val = "None";
                    sceneBool = true;
                    HandleLoadPreset("");
                });

                selectPerson = new JSONStorableStringChooser("Select Person", null, "None", "Select Person");
                UIDynamicPopup udp3 = CreatePopup(selectPerson, true);
                udp3.popup.selectColor = new Color(0.8f, 0.8f, 0.8f);
                personList.Add("None");
                selectPerson.choices = personList;
                selectPerson.val = "None";

                var loadScene = CreateButton("Load Appearance From Scene", true);
                loadScene.button.onClick.AddListener(() =>
                {
                    if (selectPerson.val == "None")
                    {
                        if (scenePreviousFolder == "" | !keepFolder_Bool.val)
                        {
                            scenePreviousFolder = _jsonScenePath;
                        }
                        filename.val = "";
                        sceneBool = true;
                        var jsonSceneShortcuts = FileManagerSecure.GetShortCutsForDirectory(scenePreviousFolder, false, false, true, true);
                        if (!shortcuts_Bool.val)
                        {
                            jsonSceneShortcuts = null;
                        }
                        SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, "json", scenePreviousFolder, true, true, false, null, false, jsonSceneShortcuts);
                        //SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, "json", _jsonScenePath);
                    }
                    else
                    {
                        sceneBool = true;
                        HandleLoadPreset(reload);
                    }
                });
                JSONStorableAction LoadSceneDatas = new JSONStorableAction("Load Appearance From Scene", () =>
                {
                    sceneBool = true;
                    HandleLoadPreset("");
                });

                //spacer = CreateSpacer(false);
                //spacer.height = 40f;

                message = new JSONStorableString("", "");
                messageBox = CreateTextField(message, false);
                messageBox.height = 145f;

                hairAndClothing = new JSONStorableString("", "");
                messageBox2 = CreateTextField(hairAndClothing, false);
                messageBox2.height = 450f;

                persons = new JSONStorableString("", "");
                messageBox3 = CreateTextField(persons, true);
                messageBox3.height = 205f;

                missingItem = new JSONStorableString("", "");
                messageBox4 = CreateTextField(missingItem, true);
                messageBox4.height = 620f;

                morphs_Bool = new JSONStorableBool("Change Morphs", true);
                morphs_toggle = CreateToggle(morphs_Bool, false);
                textures_Bool = new JSONStorableBool("Change Textures", true);
                textures_toggle = CreateToggle(textures_Bool, false);
                eye_Bool = new JSONStorableBool("Change Eyes", true);
                eye_toggle = CreateToggle(eye_Bool, false);
                mouth_Bool = new JSONStorableBool("Change Mouth", true);
                mouth_toggle = CreateToggle(mouth_Bool, false);
                hair_Bool = new JSONStorableBool("Change Hair", true);
                hair_toggle = CreateToggle(hair_Bool, false);
                clothing_Bool = new JSONStorableBool("Change Clothing", true);
                clothing_toggle = CreateToggle(clothing_Bool, false);

                RegisterString(filename);
                RegisterAction(LoadVapDatas);
                RegisterAction(LoadJsonDatas);
                RegisterAction(reloadAct);
                RegisterAction(SelectSceneDatas);
                RegisterAction(LoadSceneDatas);
                RegisterBool(shortcuts_Bool);
                RegisterStringChooser(selectPerson);
                RegisterBool(morphs_Bool);
                RegisterBool(textures_Bool);
                RegisterBool(eye_Bool);
                RegisterBool(mouth_Bool);
                RegisterBool(hair_Bool);
                RegisterBool(clothing_Bool);
                RegisterBool(keepFolder_Bool);
                #endregion
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void makeDic()
        {
            idDic = new Dictionary<string, List<string>>();
            foreach (var item in idList)
            {
                var act = TargetAtom.GetStorableByID(item).GetActionNames();
                var actAndParam = TargetAtom.GetStorableByID(item).GetAllParamAndActionNames();
                List<string> param = new List<string>();
                foreach (var dummy in actAndParam)
                {
                    if (!act.Contains(dummy) && !actionList.Contains(dummy))
                    {
                        param.Add(dummy);
                    }
                }
                idDic[item] = param;
            }
        }

        #region Preset Load and Save
        private void HandleLoadPreset(string path)
        {
            if (filename.val != "")
            {
                if (filename.val.Length > 5)
                {
                    if (filename.val.Substring(filename.val.Length - 5, 5) == ".json")
                    {
                        ext = ".json";
                    }
                }

                if (filename.val.Length > 4)
                {
                    if (filename.val.Substring(filename.val.Length - 4, 4) == ".vap")
                    {
                        ext = ".vap";
                    }
                }

                if (ext == "")
                {
                    filename.val = filename.val + ".vap";
                }

                if (filename.val.IndexOf(":") == -1)
                {
                    if (filename.val.Substring(0, 7) != "Custom/")
                    {
                        path = "Custom/Atom/Person/Appearance/" + filename.val;
                    }
                    else
                    {
                        path = filename.val;
                    }
                }
                else
                {
                    path = filename.val;
                }
            }

            if (path == "")
            {
                return;
            }

            if (path != "")
            {
                reload = path;
            }

            if (!sceneBool)
            {
                personList = new List<string>();
                personList.Clear();
                personList.Add("None");
                selectPerson.choices = personList;
                selectPerson.val = "None";
                persons.val = "";
            }

            if (sceneBool && selectPerson.val == "None")
            {
                personList = new List<string>();
                personList.Clear();
                personList.Add("None");
                persons.val = "";
            }

            hairAndClothing.val = "";
            loadDir = "";
            string[] dirList = path.Split('/');
            //bool tempBool = false;
            for (int i = 0; i < dirList.Count() - 1; i++)
            {
                loadDir = loadDir + dirList[i] + "/";
            }

            string tempPath = loadDir.Substring(0, loadDir.Length - 1);
            var charPosi = tempPath.IndexOf("Custom/Atom/Person/");
            if (charPosi != -1)
            {
                vapPreviousFolder = tempPath.Substring(charPosi);
                vapPreviousFolder = vapPreviousFolder.Replace("/", "\\");
                //SuperController.LogMessage(vapPreviousFolder);
            }

            charPosi = tempPath.IndexOf("Saves/Person/");
            if (charPosi != -1)
            {
                jsonPreviousFolder = tempPath.Substring(charPosi);
                jsonPreviousFolder = jsonPreviousFolder.Replace("/", "\\");
                //SuperController.LogMessage(jsonPreviousFolder);
            }

            charPosi = tempPath.IndexOf("Saves/scene/");
            if (charPosi != -1)
            {
                scenePreviousFolder = tempPath.Substring(charPosi);
                scenePreviousFolder = scenePreviousFolder.Replace("/", "\\");
                //SuperController.LogMessage(scenePreviousFolder);
            }

            if (path.Contains(":"))
            {
                SELF = path.Substring(0, path.IndexOf(":"));
            }
            //SuperController.LogMessage(SELF);
            //SuperController.LogMessage(path);

            LoadData = SuperController.singleton.ReadFileIntoString(path);
            jSONNode = JSON.Parse(LoadData);
            bool jsonExt = (path.Substring(path.Length - 4, 4) != ".vap");
            if (jsonExt)
            {
                atomArray = jSONNode["atoms"].AsArray;
                for (int k = 0; k < atomArray.Count; k++)
                {
                    asAtomObject = atomArray[k].AsObject;
                    id = asAtomObject["id"];
                    string type = asAtomObject["type"];
                    JSONArray tempArray = asAtomObject["storables"].AsArray;
                    JSONClass tempObject;
                    string characterType = "";
                    for (int l = 0; l < tempArray.Count; l++)
                    {
                        tempObject = tempArray[l].AsObject;
                        string tempId = tempObject["id"];
                        if ("geometry" == tempId)
                        {
                            if (tempObject["character"] != null)
                            {
                                characterType = tempObject["character"];
                            }
                        }
                    }

                    if (type == "Person")
                    {
                        if (sceneBool && selectPerson.val == "None")
                        {
                            personList.Add(id);
                            persons.val = persons.val + id + " :  " + characterType + "\n";
                        }
                        asArray = asAtomObject["storables"].AsArray;
                    }
                }

                if (sceneBool && selectPerson.val == "None")
                {
                    if (personList.Count != 2)
                    {
                        selectPerson.choices = personList;
                        return;
                    }
                    else
                    {
                        selectPerson.choices = personList;
                        selectPerson.val = personList[1];
                    }
                }

                if (sceneBool)
                {
                    for (int k = 0; k < atomArray.Count; k++)
                    {
                        asAtomObject = atomArray[k].AsObject;
                        id = asAtomObject["id"];
                        string type = asAtomObject["type"];
                        if (type == "Person" && id == selectPerson.val)
                        {
                            asArray = asAtomObject["storables"].AsArray;
                        }
                    }
                }
            }
            else
            {
                asArray = jSONNode["storables"].AsArray;
            }

            idListVap = new List<string>();

            for (int i = 0; i < asArray.Count; i++)
            {
                asObject = asArray[i].AsObject;
                id = asObject["id"];
                idListVap.Add(id);
                if (id == "geometry" && asObject["character"] != null)
                {
                    characterName = asObject["character"];
                }

                if (id == "geometry" && asObject["hair"] != null)
                {
                    hairList = new List<string>();
                    if (asObject["hair"].Count != 0)
                    {
                        hairAsArray = asObject["hair"].AsArray;
                        for (int j = 0; j < hairAsArray.Count; j++)
                        {
                            hairAsObject = hairAsArray[j].AsObject;
                            if (hairAsObject["id"] != null)
                            {
                                string hair = hairAsObject["id"];
                                if (hair.Substring(hair.Length - 4) == ".vam")
                                {
                                    if (hairAsObject["internalId"] != null && hairAsObject["internalId"] != "")
                                    {
                                        hairList.Add(hairAsObject["internalId"]);
                                    }
                                }
                                else
                                {
                                    if (hairAsObject["id"] != null && hairAsObject["id"] != "")
                                    {
                                        hairList.Add(hairAsObject["id"]);
                                    }
                                }
                            }
                            else
                            {
                                if (hairAsObject["name"] != null && hairAsObject["name"] != "")
                                {
                                    hairList.Add(hairAsObject["name"]);
                                }
                            }
                        }
                    }
                    else
                    {
                        if (asObject["hair"] != null && asObject["hari"] != "")
                        {
                            hairList.Add(asObject["hair"]);
                        }
                    }
                    hairList.Add("HairPresets");
                    hairList.Add("hairScalpMaskTool");
                    hairList.Add("HairSim");
                    hairList.Add("hairTool");
                    hairList.Add("Sim2Hair");
                    hairList.Add("V5ElitePonytail");
                    hairList.Add("VictoriaElitePonytail");
                    //hairList.Add("HairSimShortBangs");

                    //SuperController.LogMessage("hair");
                }

                if (id == "geometry" && asObject["clothing"] != null)
                {
                    clothingList = new List<string>();
                    if (asObject["clothing"].Count != 0)
                    {
                        clothingAsArray = asObject["clothing"].AsArray;
                        for (int j = 0; j < clothingAsArray.Count; j++)
                        {
                            clothingAsObject = clothingAsArray[j].AsObject;
                            if (clothingAsObject["id"] != null)
                            {
                                string clothing = clothingAsObject["id"];
                                if (clothing.Substring(clothing.Length - 4) == ".vam")
                                {
                                    clothingList.Add(clothingAsObject["internalId"]);
                                    string temp = clothingAsObject["internalId"];
                                    temp = temp.Replace(" ", "");
                                    clothingList.Add(temp);
                                }
                                else
                                {
                                    clothingList.Add(clothingAsObject["id"]);
                                    string temp = clothingAsObject["id"];
                                    temp = temp.Replace(" ", "");
                                    clothingList.Add(temp);
                                }
                            }
                            else if (clothingAsObject["name"] != null)
                            {
                                string clothing = clothingAsObject["name"];
                                if (clothing.Substring(clothing.Length - 4) == ".vam")
                                {
                                    clothingList.Add(clothingAsObject["internalId"]);
                                    string temp = clothingAsObject["internalId"];
                                    temp = temp.Replace(" ", "");
                                    clothingList.Add(temp);
                                }
                                else
                                {
                                    clothingList.Add(clothingAsObject["name"]);
                                    string temp = clothingAsObject["name"];
                                    temp = temp.Replace(" ", "");
                                    clothingList.Add(temp);
                                }
                            }
                        }
                    }

                    foreach (var item in clothingList)
                    {
                        //SuperController.LogMessage(item);
                    }

                    //SuperController.LogMessage("clothing");
                }

                if (id == "rescaleObject")
                {
                    if (asObject["scale"] != null)
                    {
                        scale = float.Parse(asObject["scale"]);
                    }
                    else
                    {
                        scale = 1f;
                    }
                }
            }

            geometry = TargetAtom.GetStorableByID("geometry");
            character = geometry as DAZCharacterSelector;
            morphControl = character.morphsControlUI;

            paramControls = new Dictionary<string, JSONStorable>();
            foreach (var item in idList)
            {
                paramControls[item] = TargetAtom.GetStorableByID(item);
            }
            makeDic();

            message.val = "Processing";
            missingItem.val = "";

            StartCoroutine(applyData());
        }

        IEnumerator applyData()
        {
            StartCoroutine(setNewCharacter());
            message.val = "Set New Character";
            yield return new WaitForSeconds(1f);
            clearOldSkin();
            message.val = "Clear Old Textures";
            yield return new WaitForSeconds(1f);
            setNewTextures();
            message.val = "Set New Textures";
            yield return new WaitForSeconds(1f);
            clearOldData();
            message.val = "Clear Old Data";
            yield return new WaitForSeconds(1f);
            setNewData();
            message.val = "Finished Morphs Copy";
            yield return new WaitForSeconds(1f);
            clearOldHair();
            message.val = "Clear Old Hair";
            //yield return new WaitForSeconds(1f);
            setNewHair1();
            message.val = "Set New Hair";
            yield return new WaitForSeconds(1f);
            setNewHair2();
            message.val = "Finished Hair Copy";
            yield return new WaitForSeconds(1f);
            clearOldClothing();
            message.val = "Clear Old Clothing";
            setNewClothing1();
            message.val = "Set New Clothing";
            yield return new WaitForSeconds(1f);
            setNewClothing2();
            message.val = "Finished Clothing copy";
            yield return new WaitForSeconds(1f);
            StartCoroutine(setNewTextures2());
            message.val = "Finished Textures Copy";
            yield return new WaitForSeconds(1f);
            setNewSkin();
            yield return new WaitForSeconds(5f);
            simEnable();
            TargetAtom.collisionEnabledJSON.val = true;
            message.val = "Finished";
        }

        private void clearOldData()
        {
            if (morphs_Bool.val)
            {
                geometry.GetBoolJSONParam("useAdvancedColliders").SetValToDefault();
                geometry.GetBoolJSONParam("useAuxBreastColliders").SetValToDefault();
                geometry.GetBoolJSONParam("disableAnatomy").SetValToDefault();
                geometry.GetBoolJSONParam("useMaleMorphsOnFemale").SetValToDefault();
                geometry.GetBoolJSONParam("useFemaleMorphsOnMale").SetValToDefault();

                id = "BreastControl";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                }

                id = "GluteControl";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                }

                id = "LowerPhysicsMesh";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                }

                id = "BreastPhysicsMesh";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                }

                id = "JawControl";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                }

                TargetAtom.GetStorableByID("BendFix").GetBoolJSONParam("enabled").SetValToDefault();
                TargetAtom.GetStorableByID("FemaleAnatomy").GetBoolJSONParam("enabled").SetValToDefault();
                TargetAtom.GetStorableByID("BreastInOut").GetBoolJSONParam("enabled").SetValToDefault();
                TargetAtom.GetStorableByID("EyelidControl").GetBoolJSONParam("eyelidLookMorphsEnabled").SetValToDefault();
                TargetAtom.GetStorableByID("AutoJawMouthMorph").GetBoolJSONParam("enabled").SetValToDefault();
                //TargetAtom.GetStorableByID("AutoExpressions").GetBoolJSONParam("enabled").SetValToDefault();
                TargetAtom.GetStorableByID("SoftBodyPhysicsEnabler").GetBoolJSONParam("enabled").SetValToDefault();
            }

            if (textures_Bool.val)
            {
                id = "FemaleEyelashes";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetColorJSONParam(item) != null)
                    {
                        paramControls[id].GetColorJSONParam(item).SetValToDefault();
                    }
                }

                id = "MaleEyelashes";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetColorJSONParam(item) != null)
                    {
                        paramControls[id].GetColorJSONParam(item).SetValToDefault();
                    }
                }
            }

            if (eye_Bool.val)
            {
                id = "irises";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetUrlJSONParam(item) != null)
                    {
                        paramControls[id].GetUrlJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetColorJSONParam(item) != null)
                    {
                        paramControls[id].GetColorJSONParam(item).SetValToDefault();
                    }
                }

                id = "sclera";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetUrlJSONParam(item) != null)
                    {
                        paramControls[id].GetUrlJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetColorJSONParam(item) != null)
                    {
                        paramControls[id].GetColorJSONParam(item).SetValToDefault();
                    }
                }

                id = "lacrimals";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetUrlJSONParam(item) != null)
                    {
                        paramControls[id].GetUrlJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetColorJSONParam(item) != null)
                    {
                        paramControls[id].GetColorJSONParam(item).SetValToDefault();
                    }
                }
            }

            if (mouth_Bool.val)
            {
                id = "teeth";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetUrlJSONParam(item) != null)
                    {
                        paramControls[id].GetUrlJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetColorJSONParam(item) != null)
                    {
                        paramControls[id].GetColorJSONParam(item).SetValToDefault();
                    }
                }

                id = "tongue";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetUrlJSONParam(item) != null)
                    {
                        paramControls[id].GetUrlJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetColorJSONParam(item) != null)
                    {
                        paramControls[id].GetColorJSONParam(item).SetValToDefault();
                    }
                }

                id = "mouth";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetUrlJSONParam(item) != null)
                    {
                        paramControls[id].GetUrlJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetColorJSONParam(item) != null)
                    {
                        paramControls[id].GetColorJSONParam(item).SetValToDefault();
                    }
                }
            }
        }

        private void clearOldSkin()
        {
            if (textures_Bool.val)
            {
                id = "skin";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetUrlJSONParam(item) != null)
                    {
                        paramControls[id].GetUrlJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetStringChooserJSONParam(item) != null)
                    {
                        paramControls[id].GetStringChooserJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetColorJSONParam(item) != null)
                    {
                        paramControls[id].GetColorJSONParam(item).SetValToDefault();
                    }
                }

                id = "textures";
                foreach (var item in idDic[id])
                {
                    if (paramControls[id].GetBoolJSONParam(item) != null)
                    {
                        paramControls[id].GetBoolJSONParam(item).SetValToDefault();
                    }
                    if (paramControls[id].GetFloatJSONParam(item) != null)
                    {
                        paramControls[id].GetFloatJSONParam(item).SetValToDefault();
                    }
                    if (item.Contains("Url"))
                    {
                        paramControls[id].GetUrlJSONParam(item).SetValToDefault();
                    }
                }
                //paramControls[id].GetBoolJSONParam("autoBlendGenitalTextures").val = false;
                //paramControls[id].GetBoolJSONParam("autoBlendGenitalSpecGlossNormalTextures").val = false;
            }
        }

        private void clearOldHair()
        {
            if (hair_Bool.val)
            {
                var allHair = TargetAtom.GetStorableByID("geometry").GetBoolParamNames();
                foreach (var item in allHair)
                {
                    if (TargetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val && item.Contains("hair:"))
                    {
                        TargetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = false;
                    }
                }
            }
        }

        private void clearOldClothing()
        {
            if (clothing_Bool.val)
            {
                var allClothing = TargetAtom.GetStorableByID("geometry").GetBoolParamNames();
                foreach (var item in allClothing)
                {
                    if (TargetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val && item.Contains("clothing:"))
                    {
                        TargetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = false;
                    }
                }
            }
        }

        private void setNewHair1()
        {
            if (hair_Bool.val)
            {
                for (int i = 0; i < asArray.Count; i++)
                {
                    asObject = asArray[i].AsObject;
                    id = asObject["id"];

                    if (id == "geometry" && asObject["hair"] != null)
                    {
                        if (asObject["hair"].Count != 0)
                        {
                            hairAsArray = asObject["hair"].AsArray;
                            for (int j = 0; j < hairAsArray.Count; j++)
                            {
                                bool hit = false;
                                hairAsObject = hairAsArray[j].AsObject;
                                if (hairAsObject["id"] != null)
                                {
                                    tempHair = hairAsObject["id"];
                                }
                                else
                                {
                                    tempHair = hairAsObject["name"];
                                }

                                if (geometry.GetBoolJSONParam("hair:" + tempHair) != null)
                                {
                                    hit = true;
                                    tempHair = "hair:" + tempHair;
                                }
                                else
                                {
                                    if (tempHair.Contains(":/") && tempHair.Substring(tempHair.Length - 4) == ".vam")
                                    {
                                        string[] tempHairVar = tempHair.Split(':');
                                        string[] tempHairName = tempHairVar[0].Split('.');
                                        int ver = tempHairName.Count();
                                        string tempName = "";
                                        for (int k = 0; k < ver - 1; k++)
                                        {
                                            tempName = tempName + tempHairName[k] + ".";
                                        }

                                        string filenameVar;
                                        int varVer = 0;
                                        for (int k = 0; k < 100; k++)
                                        {
                                            filenameVar = tempName + k.ToString() + ":" + tempHairVar[1];
                                            if (geometry.GetBoolJSONParam("hair:" + filenameVar) != null)
                                            {
                                                varVer = k;
                                            }
                                        }

                                        if (varVer != 0)
                                        {
                                            hit = true;
                                            tempHair = "hair:" + tempName + varVer.ToString() + ":" + tempHairVar[1];
                                        }
                                        else
                                        {
                                            tempHair = tempHairVar[1].Substring(1);
                                            if (geometry.GetBoolJSONParam("hair:" + tempHair) != null)
                                            {
                                                hit = true;
                                                tempHair = "hair:" + tempHair;
                                            }
                                        }
                                    }

                                    if (!hit && !tempHair.Contains(":/") && tempHair.Contains(":") && !tempHair.Contains(".vam"))
                                    {
                                        string[] tempHairName = tempHair.Split(':');
                                        foreach (var item in geometry.GetBoolParamNames())
                                        {
                                            if (item.Contains(tempHairName[0]) && item.Contains("/" + tempHairName[1] + ".vam"))
                                            {
                                                hit = true;
                                                tempHair = item;
                                            }
                                        }

                                        if (!hit)
                                        {
                                            foreach (var item in geometry.GetBoolParamNames())
                                            {
                                                if (item.Contains("/" + tempHairName[1] + ".vam"))
                                                {
                                                    hit = true;
                                                    tempHair = item;
                                                }
                                            }
                                        }
                                    }

                                    if (!hit)
                                    {
                                        string filenameVar;
                                        if (tempHair.Contains("/"))
                                        {
                                            string[] tempHairName = tempHair.Split('/');
                                            filenameVar = "/" + tempHairName[tempHairName.Count() - 1];
                                        }
                                        else
                                        {
                                            filenameVar = "/" + tempHair;
                                        }

                                        if (!filenameVar.Contains(".vam"))
                                        {
                                            filenameVar += ".vam";
                                        }

                                        foreach (var item in geometry.GetBoolParamNames())
                                        {
                                            if (item.Contains(filenameVar))
                                            {
                                                hit = true;
                                                tempHair = item;
                                            }
                                        }
                                    }
                                }

                                //SuperController.LogMessage("test  " + tempHair);
                                //SuperController.LogMessage(hit.ToString());
                                //SuperController.LogMessage("");

                                if (hit && geometry.GetBoolJSONParam(tempHair) != null)
                                {
                                    if (hairAsObject["enabled"] != null)
                                    {
                                        geometry.GetBoolJSONParam(tempHair).val = bool.Parse(hairAsObject["enabled"]);
                                    }
                                    else
                                    {
                                        geometry.GetBoolJSONParam(tempHair).val = true;
                                    }

                                    if (hairAsObject["internalId"] != null)
                                    {
                                        hairAndClothing.val = hairAndClothing.val + hairAsObject["internalId"] + "\n";
                                    }
                                    else if (hairAsObject["name"] != null)
                                    {
                                        hairAndClothing.val = hairAndClothing.val + hairAsObject["name"] + "\n";
                                    }
                                    else if (hairAsObject["id"] != null)
                                    {
                                        string temp = hairAsObject["id"];
                                        if (!temp.Contains("/"))
                                        {
                                            hairAndClothing.val = hairAndClothing.val + hairAsObject["id"] + "\n";
                                        }
                                        else
                                        {
                                            string[] dir = temp.Split('/');
                                            temp = dir[dir.Count() - 1];
                                            hairAndClothing.val = hairAndClothing.val + temp + "\n";
                                        }
                                    }
                                }
                                else
                                {
                                    missingItem.val = missingItem.val + "\nMissing Hair:" + tempHair;
                                }
                            }
                        }
                        else
                        {
                            string temp = asObject["hair"];
                            if (temp.Length > 1)
                            {
                                string tempHair = "hair:" + asObject["hair"];
                                if (geometry.GetBoolJSONParam(tempHair) != null)
                                {
                                    geometry.GetBoolJSONParam(tempHair).val = true;
                                }
                                else
                                {
                                    foreach (var item in geometry.GetBoolParamNames())
                                    {
                                        if (item.Contains("/" + asObject["hair"] + ".vam"))
                                        {
                                            tempHair = item;
                                        }
                                    }
                                    geometry.GetBoolJSONParam(tempHair).val = true;
                                }
                            }
                        }
                        //SuperController.LogMessage(asObject["hair"]);
                    }
                }
            }
        }

        private void setNewHair2()
        {
            if (hair_Bool.val)
            {
                for (int i = 0; i < asArray.Count; i++)
                {
                    bool thisIsHair = false;
                    asObject = asArray[i].AsObject;
                    id = asObject["id"];

                    foreach (var item in hairList)
                    {
                        if (id.Contains(item))
                        {
                            thisIsHair = true;
                        }
                    }

                    if (thisIsHair)
                    {
                        hairAsObject = asArray[i].AsObject;

                        if (TargetAtom.GetStorableByID(id) != null)
                        {
                            var param = hairAsObject.Keys;
                            List<string> collection = new List<string>();
                            if (TargetAtom.GetStorableByID(id).GetUrlParamNames() != null)
                            {
                                collection = TargetAtom.GetStorableByID(id).GetUrlParamNames();
                                foreach (var item in collection)
                                {
                                    if (param.Contains(item) && hairAsObject[item] != null)
                                    {
                                        TargetAtom.GetStorableByID(id).GetUrlJSONParam(item).val = checkPath(hairAsObject[item]);
                                    }
                                    else if (param.Contains(item))
                                    {
                                        TargetAtom.GetStorableByID(id).GetUrlJSONParam(item).val = "";
                                    }
                                }
                            }

                            if (TargetAtom.GetStorableByID(id).GetBoolParamNames() != null)
                            {
                                collection = TargetAtom.GetStorableByID(id).GetBoolParamNames();
                                foreach (var item in collection)
                                {
                                    if (param.Contains(item) && hairAsObject[item] != null)
                                    {
                                        TargetAtom.GetStorableByID(id).GetBoolJSONParam(item).val = bool.Parse(hairAsObject[item]);
                                    }
                                    else if (param.Contains(item))
                                    {
                                        TargetAtom.GetStorableByID(id).GetBoolJSONParam(item).SetValToDefault();
                                    }
                                }
                            }

                            if (TargetAtom.GetStorableByID(id).GetColorParamNames() != null)
                            {
                                collection = TargetAtom.GetStorableByID(id).GetColorParamNames();
                                foreach (var item in collection)
                                {
                                    if (param.Contains(item) && hairAsObject[item] != null)
                                    {
                                        float h = float.Parse(asObject[item][0]);
                                        float s = float.Parse(asObject[item][1]);
                                        float v = float.Parse(asObject[item][2]);
                                        TargetAtom.GetStorableByID(id).GetColorJSONParam(item).SetColor(Color.HSVToRGB(h, s, v));
                                    }
                                    else if (param.Contains(item))
                                    {
                                        TargetAtom.GetStorableByID(id).GetColorJSONParam(item).SetValToDefault();
                                    }
                                }
                            }

                            if (TargetAtom.GetStorableByID(id).GetFloatParamNames() != null)
                            {
                                collection = TargetAtom.GetStorableByID(id).GetFloatParamNames();
                                foreach (var item in collection)
                                {
                                    if (param.Contains(item) && hairAsObject[item] != null)
                                    {
                                        TargetAtom.GetStorableByID(id).GetFloatJSONParam(item).val = float.Parse(hairAsObject[item]);
                                    }
                                    else if (param.Contains(item))
                                    {
                                        TargetAtom.GetStorableByID(id).GetFloatJSONParam(item).SetValToDefault();
                                    }
                                }
                            }

                            if (TargetAtom.GetStorableByID(id).GetStringParamNames() != null)
                            {
                                collection = TargetAtom.GetStorableByID(id).GetStringParamNames();
                                foreach (var item in collection)
                                {
                                    if (param.Contains(item) && hairAsObject[item] != null)
                                    {
                                        TargetAtom.GetStorableByID(id).GetStringJSONParam(item).val = hairAsObject[item];
                                    }
                                    else if (param.Contains(item))
                                    {
                                        TargetAtom.GetStorableByID(id).GetStringJSONParam(item).SetValToDefault();
                                    }
                                }
                            }

                            if (TargetAtom.GetStorableByID(id).GetVector3ParamNames() != null)
                            {
                                collection = TargetAtom.GetStorableByID(id).GetVector3ParamNames();
                                foreach (var item in collection)
                                {
                                    if (param.Contains(item) && hairAsObject[item] != null)
                                    {
                                        float x = float.Parse(asObject[item][0]);
                                        float y = float.Parse(asObject[item][1]);
                                        float z = float.Parse(asObject[item][2]);
                                        TargetAtom.GetStorableByID(id).GetVector3JSONParam(item).val = new Vector3(x, y, z);
                                    }
                                    else if (param.Contains(item))
                                    {
                                        TargetAtom.GetStorableByID(id).GetVector3JSONParam(item).SetValToDefault();
                                    }
                                }
                            }

                            if (TargetAtom.GetStorableByID(id).GetStringChooserParamNames() != null)
                            {
                                collection = TargetAtom.GetStorableByID(id).GetStringChooserParamNames();
                                foreach (var item in collection)
                                {
                                    if (param.Contains(item) && hairAsObject[item] != null)
                                    {
                                        TargetAtom.GetStorableByID(id).GetStringChooserJSONParam(item).val = hairAsObject[item];
                                    }
                                    else if (param.Contains(item))
                                    {
                                        TargetAtom.GetStorableByID(id).GetStringChooserJSONParam(item).SetValToDefault();
                                    }
                                }
                            }

                            if (TargetAtom.GetStorableByID(id).GetPresetFilePathActionNames() != null)
                            {
                                collection = TargetAtom.GetStorableByID(id).GetPresetFilePathActionNames();
                                foreach (var item in collection)
                                {
                                    if (param.Contains(item))
                                    {
                                        SuperController.LogMessage("No Hit in Hair item  :  " + item);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        private void setNewClothing1()
        {
            if (clothing_Bool.val)
            {
                for (int i = 0; i < asArray.Count; i++)
                {
                    asObject = asArray[i].AsObject;
                    id = asObject["id"];

                    if (id == "geometry" && asObject["clothing"] != null)
                    {
                        if (asObject["clothing"].Count != 0)
                        {
                            clothingAsArray = asObject["clothing"].AsArray;
                            for (int j = 0; j < clothingAsArray.Count; j++)
                            {
                                bool hit = false;
                                clothingAsObject = clothingAsArray[j].AsObject;
                                if (clothingAsObject["id"] != null)
                                {
                                    tempClothing = clothingAsObject["id"];
                                }
                                else
                                {
                                    tempClothing = clothingAsObject["name"];
                                }

                                if (geometry.GetBoolJSONParam("clothing:" + tempClothing) != null)
                                {
                                    hit = true;
                                    tempClothing = "clothing:" + tempClothing;
                                }
                                else
                                {
                                    if (tempClothing.Contains(":/") && tempClothing.Substring(tempClothing.Length - 4) == ".vam")
                                    {
                                        string[] tempClothingVar = tempClothing.Split(':');
                                        string[] tempClothingName = tempClothingVar[0].Split('.');
                                        int ver = tempClothingName.Count();
                                        string tempName = "";
                                        for (int k = 0; k < ver - 1; k++)
                                        {
                                            tempName = tempName + tempClothingName[k] + ".";
                                        }

                                        string filenameVar;
                                        int varVer = 0;
                                        for (int k = 0; k < 100; k++)
                                        {
                                            filenameVar = tempName + k.ToString() + ":" + tempClothingVar[1];
                                            if (geometry.GetBoolJSONParam("clothing:" + filenameVar) != null)
                                            {
                                                varVer = k;
                                            }
                                        }

                                        if (varVer != 0)
                                        {
                                            hit = true;
                                            tempClothing = "clothing:" + tempName + varVer.ToString() + ":" + tempClothingVar[1];
                                        }
                                        else
                                        {
                                            tempClothing = tempClothingVar[1].Substring(1);
                                            if (geometry.GetBoolJSONParam("clothing:" + tempClothing) != null)
                                            {
                                                hit = true;
                                                tempClothing = "clothing:" + tempClothing;
                                            }
                                        }
                                    }

                                    if (!hit && tempClothing.Contains(":") && !tempClothing.Contains(".vam"))
                                    {
                                        string[] tempClothingName = tempClothing.Split(':');

                                        foreach (var item in geometry.GetBoolParamNames())
                                        {
                                            if (item.Contains(tempClothingName[0]) && item.Contains("/" + tempClothingName[1] + ".vam"))
                                            {
                                                hit = true;
                                                tempClothing = item;
                                            }
                                        }

                                        if (!hit)
                                        {
                                            foreach (var item in geometry.GetBoolParamNames())
                                            {
                                                if (item.Contains("/" + tempClothingName[1] + ".vam"))
                                                {
                                                    hit = true;
                                                    tempClothing = item;
                                                }
                                            }
                                        }
                                    }

                                    if (!hit)
                                    {
                                        string filenameVar;
                                        if (tempClothing.Contains("/"))
                                        {
                                            string[] tempClothingName = tempClothing.Split('/');
                                            filenameVar = "/" + tempClothingName[tempClothingName.Count() - 1];
                                        }
                                        else
                                        {
                                            filenameVar = "/" + tempClothing;
                                        }

                                        if (!filenameVar.Contains(".vam"))
                                        {
                                            filenameVar += ".vam";
                                        }

                                        foreach (var item in geometry.GetBoolParamNames())
                                        {
                                            if (item.Contains(filenameVar))
                                            {
                                                hit = true;
                                                tempClothing = item;
                                            }
                                        }
                                    }
                                }

                                //SuperController.LogMessage("test  " + tempClothing);
                                //SuperController.LogMessage(hit.ToString());
                                //SuperController.LogMessage("");

                                if (hit && geometry.GetBoolJSONParam(tempClothing) != null)
                                {
                                    if (clothingAsObject["enabled"] != null)
                                    {
                                        geometry.GetBoolJSONParam(tempClothing).val = bool.Parse(clothingAsObject["enabled"]);
                                    }
                                    else
                                    {
                                        geometry.GetBoolJSONParam(tempClothing).val = true;
                                    }

                                    if (clothingAsObject["internalId"] != null)
                                    {
                                        hairAndClothing.val = hairAndClothing.val + clothingAsObject["internalId"] + "\n";
                                    }
                                    else if (clothingAsObject["name"] != null)
                                    {
                                        hairAndClothing.val = hairAndClothing.val + clothingAsObject["name"] + "\n";
                                    }
                                    else if (clothingAsObject["id"] != null)
                                    {
                                        string temp = clothingAsObject["id"];
                                        if (!temp.Contains("/"))
                                        {
                                            hairAndClothing.val = hairAndClothing.val + clothingAsObject["id"] + "\n";
                                        }
                                        else
                                        {
                                            string[] dir = temp.Split('/');
                                            temp = dir[dir.Count() - 1];
                                            hairAndClothing.val = hairAndClothing.val + temp + "\n";
                                        }
                                    }
                                }
                                else
                                {
                                    missingItem.val = missingItem.val + "\nMissing Clothing:" + tempClothing;
                                }
                            }
                        }
                        else
                        {
                            string temp = asObject["clothing"];
                            if (temp.Length > 1)
                            {
                                string tempClothing = "clothing:" + asObject["clothing"];
                                if (geometry.GetBoolJSONParam(tempClothing) != null)
                                {
                                    geometry.GetBoolJSONParam(tempClothing).val = true;
                                }
                                else
                                {
                                    foreach (var item in geometry.GetBoolParamNames())
                                    {
                                        if (item.Contains("/" + asObject["clothing"] + ".vam"))
                                        {
                                            tempClothing = item;
                                        }
                                    }
                                    geometry.GetBoolJSONParam(tempClothing).val = true;
                                }
                            }
                        }
                    }
                }

                simClothingDic = new Dictionary<string, bool>();
                simClothingList = new List<string>();
                var tempId = TargetAtom.GetStorableIDs();
                foreach (var item in tempId)
                {
                    if (TargetAtom.GetStorableByID(item).GetBoolParamNames() != null)
                    {
                        var tempBool = TargetAtom.GetStorableByID(item).GetBoolParamNames();
                        if (tempBool.Contains("simEnabled"))
                        {
                            simClothingList.Add(item);
                            simClothingDic[item] = TargetAtom.GetStorableByID(item).GetBoolJSONParam("simEnabled").defaultVal;
                            TargetAtom.GetStorableByID(item).GetBoolJSONParam("simEnabled").val = false;
                        }
                    }
                }
            }
        }

        private void setNewClothing2()
        {
            if (clothing_Bool.val)
            {
                var rigid = TargetAtom.rigidbodies;
                List<string> rigidList = new List<string>();
                foreach (var item in rigid)
                {
                    rigidList.Add(item.name);
                }
                foreach (var item in idList)
                {
                    rigidList.Add(item);
                }
                foreach (var item in idList2)
                {
                    rigidList.Add(item);
                }
                foreach (var item in idList3)
                {
                    rigidList.Add(item);
                }

                for (int i = 0; i < asArray.Count; i++)
                {
                    bool thisIsClothing = true;
                    asObject = asArray[i].AsObject;
                    id = asObject["id"];

                    if (rigidList.Contains(id) | id.Contains("plugin#") | id.Contains("Animation") | id.Contains("animation"))
                    {
                        thisIsClothing = false;
                    }

                    foreach (var item in hairList)
                    {
                        if (item != "")
                        {
                            if (id.Contains(item))
                            {
                                thisIsClothing = false;
                            }
                        }
                    }

                    //thisIsClothing = false;
                    foreach (var item in clothingList)
                    {
                        if (id.Contains(item))
                        {
                            thisIsClothing = true;
                        }
                    }

                    if (thisIsClothing)
                    {
                        clothingAsObject = asArray[i].AsObject;

                        if (TargetAtom.GetStorableByID(id) != null)
                        {
                            var param = clothingAsObject.Keys;
                            {
                                List<string> collection = new List<string>();
                                if (TargetAtom.GetStorableByID(id).GetUrlParamNames() != null)
                                {
                                    collection = TargetAtom.GetStorableByID(id).GetUrlParamNames();
                                    //SuperController.LogMessage("Url");
                                    foreach (var item in collection)
                                    {
                                        if (param.Contains(item) && clothingAsObject[item] != null)
                                        {
                                            TargetAtom.GetStorableByID(id).GetUrlJSONParam(item).val = checkPath(clothingAsObject[item]);
                                        }
                                        else if (param.Contains(item))
                                        {
                                            TargetAtom.GetStorableByID(id).GetUrlJSONParam(item).val = "";
                                        }
                                    }
                                }

                                if (TargetAtom.GetStorableByID(id).GetBoolParamNames() != null)
                                {
                                    collection = TargetAtom.GetStorableByID(id).GetBoolParamNames();
                                    //SuperController.LogMessage("Bool");
                                    foreach (var item in collection)
                                    {
                                        if (param.Contains(item) && clothingAsObject[item] != null)
                                        {
                                            if (item == "simEnabled")
                                            {
                                                //TargetAtom.GetStorableByID(id).GetBoolJSONParam(item).val = false;
                                                simClothingDic[item] = bool.Parse(clothingAsObject[item]);
                                                //SuperController.LogMessage(item + "  cloth2  " + clothingAsObject[item].ToString());
                                            }
                                            else
                                            {
                                                TargetAtom.GetStorableByID(id).GetBoolJSONParam(item).val = bool.Parse(clothingAsObject[item]);
                                            }

                                        }
                                        else if (param.Contains(item))
                                        {
                                            if (item == "simEnabled")
                                            {
                                                //TargetAtom.GetStorableByID(id).GetBoolJSONParam(item).val = false;
                                                //simClothingDic[item] = true;
                                            }
                                            else
                                            {
                                                TargetAtom.GetStorableByID(id).GetBoolJSONParam(item).SetValToDefault();
                                            }
                                        }
                                    }
                                }

                                if (TargetAtom.GetStorableByID(id).GetColorParamNames() != null)
                                {
                                    collection = TargetAtom.GetStorableByID(id).GetColorParamNames();
                                    //SuperController.LogMessage("Color");
                                    foreach (var item in collection)
                                    {
                                        if (param.Contains(item) && clothingAsObject[item] != null)
                                        {
                                            float h = float.Parse(asObject[item][0]);
                                            float s = float.Parse(asObject[item][1]);
                                            float v = float.Parse(asObject[item][2]);
                                            TargetAtom.GetStorableByID(id).GetColorJSONParam(item).SetColor(Color.HSVToRGB(h, s, v));
                                        }
                                        else if (param.Contains(item))
                                        {
                                            TargetAtom.GetStorableByID(id).GetColorJSONParam(item).SetValToDefault();
                                        }
                                    }
                                }

                                if (TargetAtom.GetStorableByID(id).GetFloatParamNames() != null)
                                {
                                    collection = TargetAtom.GetStorableByID(id).GetFloatParamNames();
                                    //SuperController.LogMessage("Float");
                                    foreach (var item in collection)
                                    {
                                        if (param.Contains(item) && clothingAsObject[item] != null)
                                        {
                                            TargetAtom.GetStorableByID(id).GetFloatJSONParam(item).val = float.Parse(clothingAsObject[item]);
                                        }
                                        else if (param.Contains(item))
                                        {
                                            TargetAtom.GetStorableByID(id).GetFloatJSONParam(item).SetValToDefault();
                                        }
                                    }
                                }

                                if (TargetAtom.GetStorableByID(id).GetStringParamNames() != null)
                                {
                                    collection = TargetAtom.GetStorableByID(id).GetStringParamNames();
                                    //SuperController.LogMessage("String");
                                    foreach (var item in collection)
                                    {
                                        if (param.Contains(item) && clothingAsObject[item] != null)
                                        {
                                            TargetAtom.GetStorableByID(id).GetStringJSONParam(item).val = clothingAsObject[item];
                                        }
                                        else if (param.Contains(item))
                                        {
                                            TargetAtom.GetStorableByID(id).GetStringJSONParam(item).SetValToDefault();
                                        }
                                    }
                                }

                                if (TargetAtom.GetStorableByID(id).GetVector3ParamNames() != null)
                                {
                                    collection = TargetAtom.GetStorableByID(id).GetVector3ParamNames();
                                    //SuperController.LogMessage("Vector3");
                                    foreach (var item in collection)
                                    {
                                        if (param.Contains(item) && clothingAsObject[item] != null)
                                        {
                                            float x = float.Parse(asObject[item][0]);
                                            float y = float.Parse(asObject[item][1]);
                                            float z = float.Parse(asObject[item][2]);
                                            TargetAtom.GetStorableByID(id).GetVector3JSONParam(item).val = new Vector3(x, y, z);
                                        }
                                        else if (param.Contains(item))
                                        {
                                            TargetAtom.GetStorableByID(id).GetVector3JSONParam(item).SetValToDefault();
                                        }
                                    }
                                }

                                if (TargetAtom.GetStorableByID(id).GetStringChooserParamNames() != null)
                                {
                                    collection = TargetAtom.GetStorableByID(id).GetStringChooserParamNames();
                                    //SuperController.LogMessage("StringChooser");
                                    foreach (var item in collection)
                                    {
                                        if (param.Contains(item) && clothingAsObject[item] != null)
                                        {
                                            TargetAtom.GetStorableByID(id).GetStringChooserJSONParam(item).val = clothingAsObject[item];
                                        }
                                        else if (param.Contains(item))
                                        {
                                            TargetAtom.GetStorableByID(id).GetStringChooserJSONParam(item).SetValToDefault();
                                        }
                                    }
                                }

                                if (TargetAtom.GetStorableByID(id).GetPresetFilePathActionNames() != null)
                                {
                                    collection = TargetAtom.GetStorableByID(id).GetPresetFilePathActionNames();
                                    //SuperController.LogMessage("Preset");
                                    foreach (var item in collection)
                                    {
                                        if (param.Contains(item))
                                        {
                                            SuperController.LogMessage("No Hit in Clothing item  :  " + item);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        //IEnumerator simEnable()
        private void simEnable()
        {
            foreach (var item in simClothingList)
            {
                TargetAtom.GetStorableByID(item).GetBoolJSONParam("simEnabled").val = simClothingDic[item];
                //TargetAtom.GetStorableByID(item).GetBoolJSONParam("simEnabled").val = true;
            }
        }

        IEnumerator setNewCharacter()
        {
            if (textures_Bool.val && characterName != "")
            {
                //character.characterSelectorUI.characterSelector.SetStringChooserParamValue("characterSelection", characterName);}
                geometry.GetStringChooserJSONParam("characterSelection").val = characterName;
                yield return new WaitForSeconds(1f);
                makeDic();

                paramControls = new Dictionary<string, JSONStorable>();
                foreach (var item in idList)
                {
                    paramControls[item] = TargetAtom.GetStorableByID(item);
                }
            }
        }

        IEnumerator changeMorph()
        {
            if (morphs_Bool.val)
            {
                if (idListVap.Contains("rescaleObject"))
                {
                    TargetAtom.GetStorableByID("rescaleObject").GetFloatJSONParam("scale").val = scale;
                }
                else
                {
                    TargetAtom.GetStorableByID("rescaleObject").GetFloatJSONParam("scale").val = 1f;
                }
                yield return new WaitForSeconds(0.1f);

                List<string> keyList = new List<string>();
                Dictionary<string, string> morphDic = new Dictionary<string, string>();

                for (int i = 0; i < asArray.Count; i++)
                {
                    asObject = asArray[i].AsObject;
                    id = asObject["id"];
                    if (id == "geometry")
                    {
                        morphsAsArray = asObject["morphs"].AsArray;
                        for (int j = 0; j < morphsAsArray.Count; j++)
                        {
                            morphs = morphsAsArray[j].AsObject;
                            if (morphs["value"] != null)
                            {
                                keyList.Add(morphs["name"]);
                                morphDic[morphs["name"]] = morphs["value"];
                            }
                        }
                    }
                }

                DAZMorph morph;
                morphControl.GetMorphUids().ForEach((uid) =>
                {
                    morph = morphControl.GetMorphByUid(uid);
                    if (!morph.isPoseControl)
                    {
                        morph.SetDefaultValue();
                    }
                });
                yield return new WaitForSeconds(1f);

                morphControl.GetMorphUids().ForEach((uid) =>
                {
                    morph = morphControl.GetMorphByUid(uid);
                    if (!morph.isPoseControl && !keyList.Contains(morph.displayName))
                    {
                        morph.SetDefaultValue();
                    }
                });
                yield return new WaitForSeconds(1f);

                foreach (var item in keyList)
                {
                    if (morphControl.GetMorphByDisplayName(item) != null)
                    {
                        if (!morphControl.GetMorphByDisplayName(item).isPoseControl)
                        {
                            morphControl.GetMorphByDisplayName(item).morphValue = float.Parse(morphDic[item]);
                            yield return new WaitForSeconds(0.02f);
                        }
                    }
                }
                yield return new WaitForSeconds(1f);
            }
        }

        private void setNewData()
        {
            StartCoroutine(changeMorph());

            for (int i = 0; i < asArray.Count; i++)
            {
                asObject = asArray[i].AsObject;
                id = asObject["id"];

                if (morphs_Bool.val)
                {
                    if (id == "geometry")
                    {
                        if (asObject["useAdvancedColliders"] != null)
                        {
                            geometry.GetBoolJSONParam("useAdvancedColliders").val = bool.Parse(asObject["useAdvancedColliders"]);
                        }

                        if (asObject["useAuxBreastColliders"] != null)
                        {
                            geometry.GetBoolJSONParam("useAuxBreastColliders").val = bool.Parse(asObject["useAuxBreastColliders"]);
                        }

                        if (asObject["disableAnatomy"] != null)
                        {
                            geometry.GetBoolJSONParam("disableAnatomy").val = bool.Parse(asObject["disableAnatomy"]);
                        }

                        if (asObject["useMaleMorphsOnFemale"] != null)
                        {
                            geometry.GetBoolJSONParam("useMaleMorphsOnFemale").val = bool.Parse(asObject["useMaleMorphsOnFemale"]);
                        }

                        if (asObject["useFemaleMorphsOnMale"] != null)
                        {
                            geometry.GetBoolJSONParam("useFemaleMorphsOnMale").val = bool.Parse(asObject["useFemaleMorphsOnMale"]);
                        }
                    }

                    if (id == "BreastControl" | id == "GluteControl" | id == "LowerPhysicsMesh" | id == "BreastPhysicsMesh" | id == "JawControl")
                    {
                        foreach (var item in idDic[id])
                        {
                            if (paramControls[id].GetBoolJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetBoolJSONParam(item).val = bool.Parse(asObject[item]);
                            }
                            else if (paramControls[id].GetFloatJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetFloatJSONParam(item).val = float.Parse(asObject[item]);
                            }
                        }
                    }

                    if (id == "BendFix" && asObject["enabled"] != null)
                    {
                        TargetAtom.GetStorableByID(id).GetBoolJSONParam("enabled").val = bool.Parse(asObject["enabled"]);
                    }
                    if (id == "FemaleAnatomy" && asObject["enabled"] != null)
                    {
                        TargetAtom.GetStorableByID(id).GetBoolJSONParam("enabled").val = bool.Parse(asObject["enabled"]);
                    }
                    if (id == "BreastInOut" && asObject["enabled"] != null)
                    {
                        TargetAtom.GetStorableByID(id).GetBoolJSONParam("enabled").val = bool.Parse(asObject["enabled"]);
                    }
                    if (id == "EyelidControl" && asObject["eyelidLookMorphsEnabled"] != null)
                    {
                        TargetAtom.GetStorableByID(id).GetBoolJSONParam("eyelidLookMorphsEnabled").val = bool.Parse(asObject["eyelidLookMorphsEnabled"]);
                    }
                    if (id == "AutoJawMouthMorph" && asObject["enabled"] != null)
                    {
                        TargetAtom.GetStorableByID(id).GetBoolJSONParam("enabled").val = bool.Parse(asObject["enabled"]);
                    }
                    if (id == "AutoExpressions" && asObject["enabled"] != null)
                    {
                        TargetAtom.GetStorableByID(id).GetBoolJSONParam("enabled").val = bool.Parse(asObject["enabled"]);
                    }
                    if (id == "SoftBodyPhysicsEnabler" && asObject["enabled"] != null)
                    {
                        TargetAtom.GetStorableByID(id).GetBoolJSONParam("enabled").val = bool.Parse(asObject["enabled"]);
                    }
                }

                if (eye_Bool.val)
                {
                    if (id == "FemaleEyelashes" | id == "FemaleEyelashes" | id == "irises" | id == "sclera" | id == "lacrimals")
                    {
                        foreach (var item in idDic[id])
                        {
                            if (paramControls[id].GetBoolJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetBoolJSONParam(item).val = bool.Parse(asObject[item]);
                            }
                            else if (paramControls[id].GetStringChooserJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetStringChooserJSONParam(item).val = asObject[item];
                            }
                            else if (paramControls[id].GetColorJSONParam(item) != null && asObject[item] != null && asObject[item].Count >= 3)
                            {
                                float h = float.Parse(asObject[item][0]);
                                float s = float.Parse(asObject[item][1]);
                                float v = float.Parse(asObject[item][2]);
                                paramControls[id].GetColorJSONParam(item).SetColor(Color.HSVToRGB(h, s, v));
                            }
                            else if (paramControls[id].GetUrlJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetUrlJSONParam(item).val = checkPath(asObject[item]);
                            }
                            else if (paramControls[id].GetFloatJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetFloatJSONParam(item).val = float.Parse(asObject[item]);
                            }
                        }
                    }
                }

                if (mouth_Bool.val)
                {
                    if (id == "teeth" | id == "tongue" | id == "mouth")
                    {
                        foreach (var item in idDic[id])
                        {
                            if (paramControls[id].GetBoolJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetBoolJSONParam(item).val = bool.Parse(asObject[item]);
                            }
                            else if (paramControls[id].GetStringChooserJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetStringChooserJSONParam(item).val = asObject[item];
                            }
                            else if (paramControls[id].GetColorJSONParam(item) != null && asObject[item] != null && asObject[item].Count >= 3)
                            {
                                float h = float.Parse(asObject[item][0]);
                                float s = float.Parse(asObject[item][1]);
                                float v = float.Parse(asObject[item][2]);
                                paramControls[id].GetColorJSONParam(item).SetColor(Color.HSVToRGB(h, s, v));
                            }
                            else if (paramControls[id].GetUrlJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetUrlJSONParam(item).val = checkPath(asObject[item]);
                            }
                            else if (paramControls[id].GetFloatJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetFloatJSONParam(item).val = float.Parse(asObject[item]);
                            }
                        }
                    }
                }
            }
        }

        private void setNewTextures()
        {
            if (textures_Bool.val)
            {
                for (int i = 0; i < asArray.Count; i++)
                {
                    asObject = asArray[i].AsObject;
                    id = asObject["id"];
                    if (id == "textures")
                    {
                        foreach (var item in idDic[id])
                        {
                            if (paramControls[id].GetUrlJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetUrlJSONParam(item).val = checkPath(asObject[item]);
                            }
                        }
                    }
                }
            }
        }

        IEnumerator setNewTextures2()
        {
            if (textures_Bool.val)
            {
                yield return new WaitForSeconds(2f);
                for (int i = 0; i < asArray.Count; i++)
                {
                    asObject = asArray[i].AsObject;
                    id = asObject["id"];
                    if (id == "textures")
                    {
                        foreach (var item in idDic[id])
                        {
                            if (paramControls[id].GetFloatJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetFloatJSONParam(item).val = float.Parse(asObject[item]);
                            }

                            if (paramControls[id].GetBoolJSONParam(item) != null && asObject[item] != null)
                            {
                                paramControls[id].GetBoolJSONParam(item).val = bool.Parse(asObject[item]);
                            }
                        }
                    }
                }
            }
        }

        private void setNewSkin()
        {
            for (int i = 0; i < asArray.Count; i++)
            {
                asObject = asArray[i].AsObject;
                id = asObject["id"];
                if (textures_Bool.val && id == "skin")
                {
                    foreach (var item in idDic[id])
                    {

                        if (paramControls[id].GetBoolJSONParam(item) != null && asObject[item] != null)
                        {
                            paramControls[id].GetBoolJSONParam(item).val = bool.Parse(asObject[item]);
                        }
                        else if (paramControls[id].GetStringChooserJSONParam(item) != null && asObject[item] != null)
                        {
                            paramControls[id].GetStringChooserJSONParam(item).val = asObject[item];
                        }
                        else if (paramControls[id].GetColorJSONParam(item) != null && asObject[item] != null && asObject[item].Count >= 3)
                        {
                            float h = float.Parse(asObject[item][0]);
                            float s = float.Parse(asObject[item][1]);
                            float v = float.Parse(asObject[item][2]);
                            paramControls[id].GetColorJSONParam(item).SetColor(Color.HSVToRGB(h, s, v));
                        }
                        else if (paramControls[id].GetUrlJSONParam(item) != null && asObject[item] != null)
                        {
                            paramControls[id].GetUrlJSONParam(item).val = checkVar(asObject[item]);
                        }
                        else if (paramControls[id].GetFloatJSONParam(item) != null && asObject[item] != null)
                        {
                            paramControls[id].GetFloatJSONParam(item).val = float.Parse(asObject[item]);
                        }
                    }
                }
            }
        }

        private string checkVar(string varFilename)
        {
            if (varFilename != "")
            {
                if (varFilename.Contains(".") && varFilename.Contains(":"))
                {
                    string[] tempFilename = varFilename.Split(':');
                    string[] splitFilename = tempFilename[0].Split('.');
                    int ver = splitFilename.Count();
                    string tempName = "";
                    string reternName = "";
                    for (int i = 0; i < ver - 1; i++)
                    {
                        tempName = tempName + splitFilename[i] + ".";
                    }

                    string filenameVar;
                    int varVer = 0;
                    for (int k = 0; k < 100; k++)
                    {
                        filenameVar = "AddonPackages/" + tempName + k.ToString() + ".var";
                        if (FileManagerSecure.FileExists(filenameVar))
                        {
                            varVer = k;
                        }
                    }
                    if (varVer == 0)
                    {
                        for (int k = 0; k < 100; k++)
                        {
                            filenameVar = "AddonPackages/AddonPackages/" + tempName + k.ToString() + ".var";
                            if (FileManagerSecure.FileExists(filenameVar))
                            {
                                varVer = k;
                            }
                        }
                    }

                    if (varVer != 0)
                    {
                        reternName = tempName + varVer.ToString() + ":" + tempFilename[1];
                        if (reternName != varFilename)
                        {
                            //SuperController.LogMessage("Change var number");
                            //SuperController.LogMessage("from " + varFilename);
                            //SuperController.LogMessage("to   " + reternName);
                            //SuperController.LogMessage("");
                        }
                        return reternName;
                    }
                }
            }
            return varFilename;
        }

        private string checkPath(string url)
        {
            string temp = url;
            if (url != "")
            {
                if (temp.Substring(0, 5) == "SELF:")
                {
                    temp = SELF + temp.Substring(4);
                }
                else if (temp.Contains(":"))
                {
                    temp = checkVar(url);
                }
                else if (temp.Substring(0, 9) == "Textures/")
                {
                    temp = "Custom/Atom/Person/" + temp;
                }
                else if (!temp.Contains("/"))
                {
                    temp = loadDir + temp;
                }
                else if (!(temp.Substring(0, 7) == "Custom/") && !(temp.Substring(0, 6) == "Saves/"))
                {
                    temp = loadDir + temp;
                }
                //SuperController.LogMessage(temp);
            }
            return temp;
        }

        void Start()
        {
            UpdateInitialMorphs();
        }

        private void UpdateInitialMorphs()
        {
            JSONStorable geometry = TargetAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;
        }
        #endregion

        public void OnDisable()
        {

        }

        public void OnDestroy()
        {

        }
    }
}

